"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CheckoutPriceCreateMetadataTypedDict = TypeAliasType(
    "CheckoutPriceCreateMetadataTypedDict", Union[str, int, bool]
)


CheckoutPriceCreateMetadata = TypeAliasType(
    "CheckoutPriceCreateMetadata", Union[str, int, bool]
)


class CheckoutPriceCreateCustomFieldDataTypedDict(TypedDict):
    r"""Key-value object storing custom field values."""


class CheckoutPriceCreateCustomFieldData(BaseModel):
    r"""Key-value object storing custom field values."""


class CheckoutPriceCreatePaymentProcessor(str, Enum):
    r"""Payment processor to use. Currently only Stripe is supported."""

    STRIPE = "stripe"


class CheckoutPriceCreateTypedDict(TypedDict):
    r"""Create a new checkout session from a product price.

    Metadata set on the checkout will be copied
    to the resulting order and/or subscription.
    """

    product_price_id: str
    r"""ID of the product price to checkout."""
    metadata: NotRequired[Dict[str, CheckoutPriceCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    custom_field_data: NotRequired[CheckoutPriceCreateCustomFieldDataTypedDict]
    r"""Key-value object storing custom field values."""
    payment_processor: CheckoutPriceCreatePaymentProcessor
    r"""Payment processor to use. Currently only Stripe is supported."""
    discount_id: NotRequired[Nullable[str]]
    r"""ID of the discount to apply to the checkout."""
    allow_discount_codes: NotRequired[bool]
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    amount: NotRequired[Nullable[int]]
    customer_name: NotRequired[Nullable[str]]
    customer_email: NotRequired[Nullable[str]]
    customer_ip_address: NotRequired[Nullable[str]]
    customer_billing_address: NotRequired[Nullable[AddressTypedDict]]
    customer_tax_id: NotRequired[Nullable[str]]
    subscription_id: NotRequired[Nullable[str]]
    r"""ID of a subscription to upgrade. It must be on a free pricing. If checkout is successful, metadata set on this checkout will be copied to the subscription, and existing keys will be overwritten."""
    success_url: NotRequired[Nullable[str]]
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""
    embed_origin: NotRequired[Nullable[str]]
    r"""If you plan to embed the checkout session, set this to the Origin of the embedding page. It'll allow the Polar iframe to communicate with the parent page."""


class CheckoutPriceCreate(BaseModel):
    r"""Create a new checkout session from a product price.

    Metadata set on the checkout will be copied
    to the resulting order and/or subscription.
    """

    product_price_id: str
    r"""ID of the product price to checkout."""

    metadata: Optional[Dict[str, CheckoutPriceCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    custom_field_data: Optional[CheckoutPriceCreateCustomFieldData] = None
    r"""Key-value object storing custom field values."""

    PAYMENT_PROCESSOR: Annotated[
        Annotated[
            CheckoutPriceCreatePaymentProcessor,
            AfterValidator(validate_const(CheckoutPriceCreatePaymentProcessor.STRIPE)),
        ],
        pydantic.Field(alias="payment_processor"),
    ] = CheckoutPriceCreatePaymentProcessor.STRIPE
    r"""Payment processor to use. Currently only Stripe is supported."""

    discount_id: OptionalNullable[str] = UNSET
    r"""ID of the discount to apply to the checkout."""

    allow_discount_codes: Optional[bool] = True
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    amount: OptionalNullable[int] = UNSET

    customer_name: OptionalNullable[str] = UNSET

    customer_email: OptionalNullable[str] = UNSET

    customer_ip_address: OptionalNullable[str] = UNSET

    customer_billing_address: OptionalNullable[Address] = UNSET

    customer_tax_id: OptionalNullable[str] = UNSET

    subscription_id: OptionalNullable[str] = UNSET
    r"""ID of a subscription to upgrade. It must be on a free pricing. If checkout is successful, metadata set on this checkout will be copied to the subscription, and existing keys will be overwritten."""

    success_url: OptionalNullable[str] = UNSET
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""

    embed_origin: OptionalNullable[str] = UNSET
    r"""If you plan to embed the checkout session, set this to the Origin of the embedding page. It'll allow the Polar iframe to communicate with the parent page."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "custom_field_data",
            "discount_id",
            "allow_discount_codes",
            "amount",
            "customer_name",
            "customer_email",
            "customer_ip_address",
            "customer_billing_address",
            "customer_tax_id",
            "subscription_id",
            "success_url",
            "embed_origin",
        ]
        nullable_fields = [
            "discount_id",
            "amount",
            "customer_name",
            "customer_email",
            "customer_ip_address",
            "customer_billing_address",
            "customer_tax_id",
            "subscription_id",
            "success_url",
            "embed_origin",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_oauth2client_ import (
    ListResourceOAuth2Client,
    ListResourceOAuth2ClientTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Oauth2ClientsListRequestTypedDict(TypedDict):
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""


class Oauth2ClientsListRequest(BaseModel):
    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""


class Oauth2ClientsListResponseTypedDict(TypedDict):
    result: ListResourceOAuth2ClientTypedDict


class Oauth2ClientsListResponse(BaseModel):
    next: Callable[[], Optional[Oauth2ClientsListResponse]]

    result: ListResourceOAuth2Client



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ExternalOrganizationSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    NAME = "name"
    MINUS_NAME = "-name"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class ProductPriceRecurringFreeCreateType(str, Enum):
    RECURRING = "recurring"


class ProductPriceRecurringFreeCreateAmountType(str, Enum):
    FREE = "free"


class ProductPriceRecurringFreeCreateTypedDict(TypedDict):
    r"""Schema to create a free recurring product price, i.e. a subscription."""

    recurring_interval: SubscriptionRecurringInterval
    type: ProductPriceRecurringFreeCreateType
    amount_type: ProductPriceRecurringFreeCreateAmountType


class ProductPriceRecurringFreeCreate(BaseModel):
    r"""Schema to create a free recurring product price, i.e. a subscription."""

    recurring_interval: SubscriptionRecurringInterval

    TYPE: Annotated[
        Annotated[
            ProductPriceRecurringFreeCreateType,
            AfterValidator(
                validate_const(ProductPriceRecurringFreeCreateType.RECURRING)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = ProductPriceRecurringFreeCreateType.RECURRING

    AMOUNT_TYPE: Annotated[
        Annotated[
            ProductPriceRecurringFreeCreateAmountType,
            AfterValidator(
                validate_const(ProductPriceRecurringFreeCreateAmountType.FREE)
            ),
        ],
        pydantic.Field(alias="amount_type"),
    ] = ProductPriceRecurringFreeCreateAmountType.FREE



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .licensekeystatus import LicenseKeyStatus
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class LicenseKeyUpdateTypedDict(TypedDict):
    status: NotRequired[Nullable[LicenseKeyStatus]]
    usage: NotRequired[int]
    limit_activations: NotRequired[Nullable[int]]
    limit_usage: NotRequired[Nullable[int]]
    expires_at: NotRequired[Nullable[datetime]]


class LicenseKeyUpdate(BaseModel):
    status: OptionalNullable[LicenseKeyStatus] = UNSET

    usage: Optional[int] = 0

    limit_activations: OptionalNullable[int] = UNSET

    limit_usage: OptionalNullable[int] = UNSET

    expires_at: OptionalNullable[datetime] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "status",
            "usage",
            "limit_activations",
            "limit_usage",
            "expires_at",
        ]
        nullable_fields = ["status", "limit_activations", "limit_usage", "expires_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class UserOrderSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    AMOUNT = "amount"
    MINUS_AMOUNT = "-amount"
    ORGANIZATION = "organization"
    MINUS_ORGANIZATION = "-organization"
    PRODUCT = "product"
    MINUS_PRODUCT = "-product"
    SUBSCRIPTION = "subscription"
    MINUS_SUBSCRIPTION = "-subscription"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk import utils
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated


class AlreadyCanceledSubscriptionError(str, Enum):
    ALREADY_CANCELED_SUBSCRIPTION = "AlreadyCanceledSubscription"


class AlreadyCanceledSubscriptionData(BaseModel):
    detail: str

    ERROR: Annotated[
        Annotated[
            AlreadyCanceledSubscriptionError,
            AfterValidator(
                validate_const(
                    AlreadyCanceledSubscriptionError.ALREADY_CANCELED_SUBSCRIPTION
                )
            ),
        ],
        pydantic.Field(alias="error"),
    ] = AlreadyCanceledSubscriptionError.ALREADY_CANCELED_SUBSCRIPTION


class AlreadyCanceledSubscription(Exception):
    data: AlreadyCanceledSubscriptionData

    def __init__(self, data: AlreadyCanceledSubscriptionData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, AlreadyCanceledSubscriptionData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceonetimecustom import (
    ProductPriceOneTimeCustom,
    ProductPriceOneTimeCustomTypedDict,
)
from .productpriceonetimefixed import (
    ProductPriceOneTimeFixed,
    ProductPriceOneTimeFixedTypedDict,
)
from .productpriceonetimefree import (
    ProductPriceOneTimeFree,
    ProductPriceOneTimeFreeTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ProductPriceOneTimeTypedDict = TypeAliasType(
    "ProductPriceOneTimeTypedDict",
    Union[
        ProductPriceOneTimeFreeTypedDict,
        ProductPriceOneTimeFixedTypedDict,
        ProductPriceOneTimeCustomTypedDict,
    ],
)


ProductPriceOneTime = TypeAliasType(
    "ProductPriceOneTime",
    Union[ProductPriceOneTimeFree, ProductPriceOneTimeFixed, ProductPriceOneTimeCustom],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ProductSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    NAME = "name"
    MINUS_NAME = "-name"
    PRICE_TYPE = "price_type"
    MINUS_PRICE_TYPE = "-price_type"
    PRICE_AMOUNT_TYPE = "price_amount_type"
    MINUS_PRICE_AMOUNT_TYPE = "-price_amount_type"
    PRICE_AMOUNT = "price_amount"
    MINUS_PRICE_AMOUNT = "-price_amount"
